use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.18

use Test::More 0.88;
use Test::EOL;

my @files = (
    'Changes',
    'GPLv3',
    'VERSION',
    'ex/MANIFEST',
    'ex/ManifestWithFileSize.out',
    'ex/ManifestWithFileSize.pm',
    'lib/Dist/Zilla/Plugin/Manifest/Write.pm',
    'lib/Dist/Zilla/Plugin/Manifest/Write/Manual.pod',
    't/CustomPlugin.pm',
    't/ManifestWriteTester.pm',
    't/examples.t',
    't/manifest-write.t',
    'xt/aspell.en.pws',
    'xt/perlcritic.ini'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
